@@ -15,13 +15,18 @@
 import importlib
 import os
 import random
+from collections.abc import Mapping
+from contextlib import contextmanager
 from dataclasses import dataclass, field
 from enum import Enum
-from typing import List, Optional, Union
+from functools import update_wrapper
+from typing import Any, List, Optional, Union
 
 import numpy as np
 import torch
 
+from packaging import version
+
 from .state import AcceleratorState, DistributedType, is_deepspeed_available, is_tpu_available
 
 
@@ -40,6 +45,11 @@ def is_sagemaker_available():
 if is_deepspeed_available():
     from deepspeed import DeepSpeedEngine
 
+SCALER_NAME = "scaler.pt"
+MODEL_NAME = "pytorch_model"
+RNG_STATE_NAME = "random_states"
+OPTIMIZER_NAME = "optimizer"
+
 
 class RNGType(Enum):
     TORCH = "torch"
@@ -54,13 +64,17 @@ class TensorInformation:
     dtype: torch.dtype
 
 
-def set_seed(seed: int):
+def set_seed(seed: int, device_specific: bool = False):
     """
-    Helper function for reproducible behavior to set the seed in ``random``, ``numpy``, ``torch``.
+    Helper function for reproducible behavior to set the seed in `random`, `numpy`, `torch`.
 
     Args:
-        seed (:obj:`int`): The seed to set.
+        seed (`int`): The seed to set.
+        device_specific (`bool`, *optional*, defaults to `False`):
+            Whether to differ the seed on each device slightly with `self.process_index`.
     """
+    if device_specific:
+        seed += AcceleratorState().process_index
     random.seed(seed)
     np.random.seed(seed)
     torch.manual_seed(seed)
@@ -87,7 +101,7 @@ def synchronize_rng_state(rng_type: Optional[RNGType] = None, generator: Optiona
     state = AcceleratorState()
     if state.distributed_type == DistributedType.TPU:
         rng_state = xm.mesh_reduce("random_seed", rng_state, lambda x: x[0])
-    elif state.distributed_type == DistributedType.MULTI_GPU:
+    elif state.distributed_type in [DistributedType.DEEPSPEED, DistributedType.MULTI_GPU]:
         rng_state = rng_state.to(state.device)
         torch.distributed.broadcast(rng_state, 0)
         rng_state = rng_state.cpu()
@@ -114,11 +128,11 @@ def honor_type(obj, generator):
     """
     Cast a generator to the same type as obj (list, tuple or namedtuple)
     """
-    # There is no direct check whether an object if of type namedtuple sadly, this is a workaround.
-    if isinstance(obj, tuple) and hasattr(obj, "_fields"):
-        # Can instantiate a namedtuple from a generator directly, contrary to a tuple/list.
+    try:
+        return type(obj)(generator)
+    except TypeError:
+        # Some objects may not be able to instantiate from a generator directly
         return type(obj)(*list(generator))
-    return type(obj)(generator)
 
 
 def is_torch_tensor(tensor):
@@ -134,23 +148,22 @@ def recursively_apply(func, data, *args, test_type=is_torch_tensor, error_on_oth
     Recursively apply a function on a data structure that is a nested list/tuple/dictionary of a given base type.
 
     Args:
-        func (:obj:`callable`):
+        func (`callable`):
             The function to recursively apply.
-        data (nested list/tuple/dictionary of :obj:`main_type`):
-            The data on which to apply :obj:`func`
+        data (nested list/tuple/dictionary of `main_type`):
+            The data on which to apply `func`
         *args:
-            Positional arguments that will be passed to :obj:`func` when applied on the unpacked data.
-        main_type (:obj:`type`, `optional`, defaults to :obj:`torch.Tensor`):
-            The base type of the objects to which apply :obj:`func`.
-        error_on_other_type (:obj:`bool`, `optional`, defaults to :obj:`False`):
-            Whether to return an error or not if after unpacking :obj:`data`, we get on an object that is not of type
-            :obj:`main_type`. If :obj:`False`, the function will leave objects of types different than :obj:`main_type`
-            unchanged.
+            Positional arguments that will be passed to `func` when applied on the unpacked data.
+        main_type (`type`, *optional*, defaults to `torch.Tensor`):
+            The base type of the objects to which apply `func`.
+        error_on_other_type (`bool`, *optional*, defaults to `False`):
+            Whether to return an error or not if after unpacking `data`, we get on an object that is not of type
+            `main_type`. If `False`, the function will leave objects of types different than `main_type` unchanged.
         **kwargs:
-            Keyword arguments that will be passed to :obj:`func` when applied on the unpacked data.
+            Keyword arguments that will be passed to `func` when applied on the unpacked data.
 
     Returns:
-        The same data structure as :obj:`data` with :obj:`func` applied to every object of type :obj:`main_type`.
+        The same data structure as `data` with `func` applied to every object of type `main_type`.
     """
     if isinstance(data, (tuple, list)):
         return honor_type(
@@ -162,9 +175,9 @@ def recursively_apply(func, data, *args, test_type=is_torch_tensor, error_on_oth
                 for o in data
             ),
         )
-    elif isinstance(data, dict):
+    elif isinstance(data, Mapping):
         return type(data)(
-            **{
+            {
                 k: recursively_apply(
                     func, v, *args, test_type=test_type, error_on_other_type=error_on_other_type, **kwargs
                 )
@@ -186,13 +199,13 @@ def send_to_device(tensor, device):
     Recursively sends the elements in a nested list/tuple/dictionary of tensors to a given device.
 
     Args:
-        tensor (nested list/tuple/dictionary of :obj:`torch.Tensor`):
+        tensor (nested list/tuple/dictionary of `torch.Tensor`):
             The data to send to a given device.
-        device (:obj:`torch.device`):
-            The device to send the data to
+        device (`torch.device`):
+            The device to send the data to.
 
     Returns:
-        The same data structure as :obj:`tensor` with all tensors sent to the proper device.
+        The same data structure as `tensor` with all tensors sent to the proper device.
     """
 
     def _send_to_device(t, device):
@@ -201,7 +214,7 @@ def send_to_device(tensor, device):
     def _has_to_method(t):
         return hasattr(t, "to")
 
-    return recursively_apply(_send_to_device, tensor, device, test_type=_has_to_method, error_on_other_type=True)
+    return recursively_apply(_send_to_device, tensor, device, test_type=_has_to_method)
 
 
 def get_data_structure(data):
@@ -209,11 +222,11 @@ def get_data_structure(data):
     Recursively gathers the information needed to rebuild a nested list/tuple/dictionary of tensors.
 
     Args:
-        data (nested list/tuple/dictionary of :obj:`torch.Tensor`):
+        data (nested list/tuple/dictionary of `torch.Tensor`):
             The data to send to analyze.
 
     Returns:
-        The same data structure as :obj:`data` with :class:`~accelerate.utils.TensorInformation` instead of tensors.
+        The same data structure as `data` with [`~utils.TensorInformation`] instead of tensors.
     """
 
     def _get_data_structure(tensor):
@@ -224,11 +237,10 @@ def get_data_structure(data):
 
 def initialize_tensors(data_structure):
     """
-    Recursively initializes tensors from a nested list/tuple/dictionary of
-    :class:`~accelerate.utils.TensorInformation`.
+    Recursively initializes tensors from a nested list/tuple/dictionary of [`~utils.TensorInformation`].
 
     Returns:
-        The same data structure as :obj:`data` with tensors instead of :class:`~accelerate.utils.TensorInformation`.
+        The same data structure as `data` with tensors instead of [`~utils.TensorInformation`].
     """
 
     def _initialize_tensor(tensor_info):
@@ -239,43 +251,52 @@ def initialize_tensors(data_structure):
 
 def convert_to_fp32(tensor):
     """
-    Recursively converts the elements nested list/tuple/dictionary of tensors in FP16 precision to FP32.
+    Recursively converts the elements nested list/tuple/dictionary of tensors in FP16/BF16 precision to FP32.
 
     Args:
-        tensor (nested list/tuple/dictionary of :obj:`torch.Tensor`):
-            The data to convert from FP16 to FP32.
+        tensor (nested list/tuple/dictionary of `torch.Tensor`):
+            The data to convert from FP16/BF16 to FP32.
 
     Returns:
-        The same data structure as :obj:`tensor` with all tensors that were in FP16 precision converted to FP32.
+        The same data structure as `tensor` with all tensors that were in FP16/BF16 precision converted to FP32.
     """
 
     def _convert_to_fp32(tensor):
         return tensor.float()
 
-    def _is_fp16_tensor(tensor):
-        return hasattr(tensor, "dtype") and tensor.dtype == torch.float16
+    def _is_fp16_bf16_tensor(tensor):
+        return hasattr(tensor, "dtype") and (
+            tensor.dtype == torch.float16
+            or (version.parse(torch.__version__) >= version.parse("1.10") and tensor.dtype == torch.bfloat16)
+        )
 
-    return recursively_apply(_convert_to_fp32, tensor, test_type=_is_fp16_tensor)
+    return recursively_apply(_convert_to_fp32, tensor, test_type=_is_fp16_bf16_tensor)
 
 
-def convert_outputs_to_fp32(model_forward):
+class ConvertOutputsToFp32:
     """
     Decorator to apply to a function outputing tensors (like a model forward pass) that ensures the outputs in FP16
     precision will be convert back to FP32.
 
+    Use a class instead of a decorator because otherwise, the prepared model can no longer be pickled (issue #273).
+
     Args:
-        model_forward (:obj:`Callable`):
+        model_forward (`Callable`):
             The function which outputs we want to treat.
 
     Returns:
-        The same function as :obj:`model_forward` but with converted outputs.
+        The same function as `model_forward` but with converted outputs.
     """
 
-    def convert_outputs(*args, **kwargs):
-        outputs = model_forward(*args, **kwargs)
-        return convert_to_fp32(outputs)
+    def __init__(self, model_forward):
+        self.model_forward = model_forward
+        update_wrapper(self, model_forward)
 
-    return convert_outputs
+    def __call__(self, *args, **kwargs):
+        return convert_to_fp32(self.model_forward(*args, **kwargs))
+
+
+convert_outputs_to_fp32 = ConvertOutputsToFp32
 
 
 def extract_model_from_parallel(model):
@@ -283,10 +304,10 @@ def extract_model_from_parallel(model):
     Extract a model from its distributed containers.
 
     Args:
-        model (:obj:`torch.nn.Module`): The model to extract.
+        model (`torch.nn.Module`): The model to extract.
 
     Returns:
-        :obj:`torch.nn.Module`: The extracted model.
+        `torch.nn.Module`: The extracted model.
     """
     options = (torch.nn.parallel.DistributedDataParallel, torch.nn.DataParallel)
     if is_deepspeed_available():
@@ -300,7 +321,7 @@ def extract_model_from_parallel(model):
 def _tpu_gather(tensor, name="gather tensor"):
     if isinstance(tensor, (list, tuple)):
         return honor_type(tensor, (_tpu_gather(t, name=f"{name}_{i}") for i, t in enumerate(tensor)))
-    elif isinstance(tensor, dict):
+    elif isinstance(tensor, Mapping):
         return type(tensor)({k: _tpu_gather(v, name=f"{name}_{k}") for k, v in tensor.items()})
     elif not isinstance(tensor, torch.Tensor):
         raise TypeError(f"Can't gather the values of type {type(tensor)}, only of nested list/tuple/dicts of tensors.")
@@ -328,15 +349,15 @@ def gather(tensor):
     Recursively gather tensor in a nested list/tuple/dictionary of tensors from all devices.
 
     Args:
-        tensor (nested list/tuple/dictionary of :obj:`torch.Tensor`):
+        tensor (nested list/tuple/dictionary of `torch.Tensor`):
             The data to gather.
 
     Returns:
-        The same data structure as :obj:`tensor` with all tensors sent to the proper device.
+        The same data structure as `tensor` with all tensors sent to the proper device.
     """
     if AcceleratorState().distributed_type == DistributedType.TPU:
         return _tpu_gather(tensor, name="accelerate.utils.gather")
-    elif AcceleratorState().distributed_type == DistributedType.MULTI_GPU:
+    elif AcceleratorState().distributed_type in [DistributedType.DEEPSPEED, DistributedType.MULTI_GPU]:
         return _gpu_gather(tensor)
     elif AcceleratorState().distributed_type == DistributedType.MULTI_CPU:
         return _cpu_gather(tensor)
@@ -344,6 +365,39 @@ def gather(tensor):
         return tensor
 
 
+def _gpu_gather_object(object: Any):
+    def _gpu_gather_object_one(object: Any):
+        output_objects = [None for _ in range(AcceleratorState().num_processes)]
+        torch.distributed.all_gather_object(output_objects, object)
+        return output_objects
+
+    return recursively_apply(_gpu_gather_object_one, object)
+
+
+_cpu_gather_object = _gpu_gather_object
+
+
+def gather_object(object: Any):
+    """
+    Recursively gather object in a nested list/tuple/dictionary of objects from all devices.
+
+    Args:
+        object (nested list/tuple/dictionary of picklable object):
+            The data to gather.
+
+    Returns:
+        The same data structure as `object` with all the objects sent to every device.
+    """
+    if AcceleratorState().distributed_type == DistributedType.TPU:
+        raise NotImplementedError("gather objects in TPU is not supported")
+    elif AcceleratorState().distributed_type in [DistributedType.DEEPSPEED, DistributedType.MULTI_GPU]:
+        return _gpu_gather_object(object)
+    elif AcceleratorState().distributed_type == DistributedType.MULTI_CPU:
+        return _cpu_gather_object(object)
+    else:
+        return object
+
+
 def _gpu_broadcast(data, src=0):
     def _gpu_broadcast_one(tensor, src=0):
         torch.distributed.broadcast(tensor, src=src)
@@ -355,7 +409,7 @@ def _gpu_broadcast(data, src=0):
 def _tpu_broadcast(tensor, src=0, name="broadcast tensor"):
     if isinstance(tensor, (list, tuple)):
         return honor_type(tensor, (_tpu_broadcast(t, name=f"{name}_{i}") for i, t in enumerate(tensor)))
-    elif isinstance(tensor, dict):
+    elif isinstance(tensor, Mapping):
         return type(tensor)({k: _tpu_broadcast(v, name=f"{name}_{k}") for k, v in tensor.items()})
     return xm.mesh_reduce(name, tensor, lambda x: x[src])
 
@@ -365,17 +419,17 @@ def broadcast(tensor, from_process: int = 0):
     Recursively broadcast tensor in a nested list/tuple/dictionary of tensors to all devices.
 
     Args:
-        tensor (nested list/tuple/dictionary of :obj:`torch.Tensor`):
+        tensor (nested list/tuple/dictionary of `torch.Tensor`):
             The data to gather.
-        from_process (:obj:`int`, `optional`, defaults to 0):
+        from_process (`int`, *optional*, defaults to 0):
             The process from which to send the data
 
     Returns:
-        The same data structure as :obj:`tensor` with all tensors broadcasted to the proper device.
+        The same data structure as `tensor` with all tensors broadcasted to the proper device.
     """
     if AcceleratorState().distributed_type == DistributedType.TPU:
         return _tpu_broadcast(tensor, src=from_process, name="accelerate.utils.broadcast")
-    elif AcceleratorState().distributed_type == DistributedType.MULTI_GPU:
+    elif AcceleratorState().distributed_type in [DistributedType.DEEPSPEED, DistributedType.MULTI_GPU]:
         return _gpu_broadcast(tensor, src=from_process)
     elif AcceleratorState().distributed_type == DistributedType.MULTI_CPU:
         return _gpu_broadcast(tensor, src=from_process)
@@ -390,7 +444,7 @@ def broadcast_object_list(object_list, from_process: int = 0):
     Args:
         object_list (list of picklable objects):
             The list of objects to broadcast. This list will be modified inplace.
-        from_process (:obj:`int`, `optional`, defaults to 0):
+        from_process (`int`, *optional*, defaults to 0):
             The process from which to send the data.
 
     Returns:
@@ -399,7 +453,7 @@ def broadcast_object_list(object_list, from_process: int = 0):
     if AcceleratorState().distributed_type == DistributedType.TPU:
         for i, obj in enumerate(object_list):
             object_list[i] = xm.mesh_reduce("accelerate.utils.broadcast_object_list", obj, lambda x: x[from_process])
-    elif AcceleratorState().distributed_type == DistributedType.MULTI_GPU:
+    elif AcceleratorState().distributed_type in [DistributedType.DEEPSPEED, DistributedType.MULTI_GPU]:
         torch.distributed.broadcast_object_list(object_list, src=from_process)
     elif AcceleratorState().distributed_type == DistributedType.MULTI_CPU:
         torch.distributed.broadcast_object_list(object_list, src=from_process)
@@ -411,13 +465,13 @@ def slice_tensors(data, tensor_slice):
     Recursively takes a slice in a nested list/tuple/dictionary of tensors.
 
     Args:
-        data (nested list/tuple/dictionary of :obj:`torch.Tensor`):
+        data (nested list/tuple/dictionary of `torch.Tensor`):
             The data to slice.
-        tensor_slice (:obj:`slice`):
+        tensor_slice (`slice`):
             The slice to take.
 
     Returns:
-        The same data structure as :obj:`data` with all the tensors slices.
+        The same data structure as `data` with all the tensors slices.
     """
 
     def _slice_tensor(tensor, tensor_slice):
@@ -431,14 +485,14 @@ def find_batch_size(data):
     Recursively finds the batch size in a nested list/tuple/dictionary of lists of tensors.
 
     Args:
-        data (nested list/tuple/dictionary of :obj:`torch.Tensor`): The data from which to find the batch size.
+        data (nested list/tuple/dictionary of `torch.Tensor`): The data from which to find the batch size.
 
     Returns:
-        :obj:`int`: The batch size.
+        `int`: The batch size.
     """
     if isinstance(data, (tuple, list)):
         return find_batch_size(data[0])
-    elif isinstance(data, dict):
+    elif isinstance(data, Mapping):
         for k in data.keys():
             return find_batch_size(data[k])
     elif not isinstance(data, torch.Tensor):
@@ -451,18 +505,18 @@ def concatenate(data, dim=0):
     Recursively concatenate the tensors in a nested list/tuple/dictionary of lists of tensors with the same shape.
 
     Args:
-        data (nested list/tuple/dictionary of lists of tensors :obj:`torch.Tensor`):
+        data (nested list/tuple/dictionary of lists of tensors `torch.Tensor`):
             The data to concatenate.
-        dim (:obj:`int`, `optional`, defaults to 0):
+        dim (`int`, *optional*, defaults to 0):
             The dimension on which to concatenate.
 
     Returns:
-        The same data structure as :obj:`data` with all the tensors concatenated.
+        The same data structure as `data` with all the tensors concatenated.
     """
     if isinstance(data[0], (tuple, list)):
         return honor_type(data[0], (concatenate([d[i] for d in data], dim=dim) for i in range(len(data[0]))))
-    elif isinstance(data[0], dict):
-        return type(data[0])(**{k: concatenate([d[k] for d in data], dim=dim) for k in data[0].keys()})
+    elif isinstance(data[0], Mapping):
+        return type(data[0])({k: concatenate([d[k] for d in data], dim=dim) for k in data[0].keys()})
     elif not isinstance(data[0], torch.Tensor):
         raise TypeError(f"Can only concatenate tensors but got {type(data[0])}")
     return torch.cat(data, dim=dim)
@@ -474,13 +528,13 @@ def pad_across_processes(tensor, dim=0, pad_index=0, pad_first=False):
     can safely be gathered.
 
     Args:
-        tensor (nested list/tuple/dictionary of :obj:`torch.Tensor`):
+        tensor (nested list/tuple/dictionary of `torch.Tensor`):
             The data to gather.
-        dim (:obj:`int`, `optional`, defaults to 0):
+        dim (`int`, *optional*, defaults to 0):
             The dimension on which to pad.
-        pad_index (:obj:`int`, `optional`, defaults to 0):
+        pad_index (`int`, *optional*, defaults to 0):
             The value with which to pad.
-        pad_first (:obj:`bool`, `optional`, defaults to :obj:`False`):
+        pad_first (`bool`, *optional*, defaults to `False`):
             Whether to pad at the beginning or the end.
     """
 
@@ -518,9 +572,11 @@ def wait_for_everyone():
     """
     Introduces a blocking point in the script, making sure all processes have reached this point before continuing.
 
-    Warning::
+    <Tip warning={true}>
 
-        Make sure all processes will reach this instruction otherwise one of your processes will hang forever.
+    Make sure all processes will reach this instruction otherwise one of your processes will hang forever.
+
+    </Tip>
     """
     if (
         AcceleratorState().distributed_type == DistributedType.MULTI_GPU
@@ -534,7 +590,7 @@ def wait_for_everyone():
 
 def save(obj, f):
     """
-    Save the data to disk. Use in place of :obj:`torch.save()`.
+    Save the data to disk. Use in place of `torch.save()`.
 
     Args:
         obj: The data to save
@@ -551,18 +607,30 @@ class PrepareForLaunch:
     Prepare a function that will launched in a distributed setup.
 
     Args:
-        launcher (:obj:`Callable`):
+        launcher (`Callable`):
             The function to launch.
-        distributed_type (:class:`~accelerate.state.DistributedType`):
+        distributed_type ([`~state.DistributedType`]):
             The distributed type to prepare for.
+        debug (`bool`, *optional*, defaults to `False`):
+            Whether or not this is a debug launch.
     """
 
-    def __init__(self, launcher, distributed_type="NO"):
+    def __init__(self, launcher, distributed_type="NO", debug=False):
         self.launcher = launcher
         self.distributed_type = DistributedType(distributed_type)
+        self.debug = debug
 
     def __call__(self, index, *args):
-        if self.distributed_type == DistributedType.MULTI_GPU or self.distributed_type == DistributedType.MULTI_CPU:
+        if self.debug:
+            world_size = int(os.environ.get("WORLD_SIZE"))
+            rdv_file = os.environ.get("ACCELERATE_DEBUG_RDV_FILE")
+            torch.distributed.init_process_group(
+                "gloo",
+                rank=index,
+                store=torch.distributed.FileStore(rdv_file, world_size),
+                world_size=world_size,
+            )
+        elif self.distributed_type == DistributedType.MULTI_GPU or self.distributed_type == DistributedType.MULTI_CPU:
             # Prepare the environment for torch.distributed
             os.environ["LOCAL_RANK"] = str(index)
             os.environ["RANK"] = str(index)
@@ -615,3 +683,32 @@ class DeepSpeedPlugin:
             "steps_per_print": float("inf"),  # this will stop deepspeed from logging @ stdout
             "zero_allow_untested_optimizer": True,
         }
+
+
+@contextmanager
+def patch_environment(**kwargs):
+    """
+    A context manager that will add each keyword argument passed to `os.environ` and remove them when exiting.
+
+    Will convert the values in `kwargs` to strings and upper-case all the keys.
+    """
+    for key, value in kwargs.items():
+        os.environ[key.upper()] = str(value)
+
+    yield
+
+    for key in kwargs:
+        del os.environ[key.upper()]
+
+
+def get_pretty_name(obj):
+    """
+    Gets a pretty name from `obj`.
+    """
+    if not hasattr(obj, "__qualname__") and not hasattr(obj, "__name__"):
+        obj = getattr(obj, "__class__", obj)
+    if hasattr(obj, "__qualname__"):
+        return obj.__qualname__
+    if hasattr(obj, "__name__"):
+        return obj.__name__
+    return str(obj)
